--[[ algorithm

	interval 1
	limit 5
	
	
	
	every interval, spawn a "brick" on screen
		-> brick is a rectangle primitive, uniform size
	
	if bricks > limit, game over
	
	user clicks on brick to destroy it
	if user has destroyed <limit * 2> bricks, decrease interval by 10%


--]]

require("brick");

-- vars
local state = 0;
local ents = {};
local interval = 1;
local limit = 5;
local tick = 0;
local score = 0;
local stage = 1;
local diffMod = 0.925;
sys = "undefined";
width = 0;
height = 0;

-- states
local STATE_START = 0;
local STATE_INGAME = 1;
local STATE_GAMEOVER = 2;
local STATE_PAUSED = 3;

-- buttons
local confirm = "menu";
local cancel = "escape";

-- assets
local bgm = bgm;

function love.load()
	width, height = love.window.getDimensions();
	bgm = love.audio.newSource("bgm.mod");
	sys = love.system.getOS();
	if sys ~= "Android" then 
		confirm = "return";
	else
		confirm = "menu";
		love.graphics.setFont(love.graphics.newFont(36));
	end
end

function love.draw()
	drawHud();
	
	if state == STATE_START then drawStart();
	elseif state == STATE_INGAME then drawGame();
	elseif state == STATE_GAMEOVER then drawGameOver();
	elseif state == STATE_PAUSED then drawPaused();
	end
	
	love.graphics.setColor(0, 0, 0, 255)
end

function love.update(dt)
	if state == STATE_START then updateStart(dt);
	elseif state == STATE_INGAME then updateGame(dt);
	elseif state == STATE_GAMEOVER then updateGameOver(dt);
	elseif state == STATE_PAUSED then updatePaused(dt);
	end
end

function love.keypressed(key)
	if state == STATE_START then inputStart(key);
	elseif state == STATE_INGAME then inputGame(key);
	elseif state == STATE_GAMEOVER then inputGameOver(key);
	elseif state == STATE_PAUSED then inputPaused(key);
	end
end

function love.mousepressed(x, y, btn)
	if state == STATE_START then mouseStart(x, y, btn);
	elseif state == STATE_INGAME then mouseGame(x, y, btn);
	elseif state == STATE_GAMEOVER then mouseGameOver(x, y, btn);
	elseif state == STATE_PAUSED then mousePaused(x, y, btn);
	end
end

function init()

	if bgm:isPlaying() == false then
		bgm = love.audio.newSource("bgm.mod");
		love.audio.play(bgm) 
	end
	--state = 0;
	interval = 1;
	limit = 5;
	ents = {};
	table.insert(ents, brick());
	tick = love.timer.getTime();
	score = 0;
	stage = 1;

end

function round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end

function drawHud()
	local total = table.getn(ents);
	if total == nil then total = 0 end
	local stats = "Bricks: " .. total .. "/" .. limit;
	--stats = stats .. "\nInterval: " .. round(interval, 2);
	stats = stats .. "\nStage: " .. stage;
	stats = stats .. "\nScore: " .. score;
	
	if (total <= 2) then love.graphics.setColor(255, 255, 255, 255) end
	if (total >= 3) then love.graphics.setColor(255, 0, 0, 255) end
	
	love.graphics.print(stats, 0, 0, 0);
end

-- states

-- STATE_START

function drawStart()

	love.graphics.setColor(255, 255, 255, 255);
	love.graphics.printf("PRESS " .. string.upper(confirm) .. " TO BEGIN!", 0, height / 2, width, "center", 0);

end

function updateStart(dt)
	-- noop
end

function inputStart(key)
	if key == confirm then 
		state = STATE_INGAME;
		init();
	elseif key == cancel then love.event.quit();
	end
end

function mouseStart(x, y, btn)
	-- noop
end

-- STATE_INGAME

function drawGame()
	-- draw all bricks
	for k, v in pairs(ents) do
		local r, g, b = v:getColour();
		local x, y, x2, y2 = v:getHitbox();
		love.graphics.setColor(r, g, b, 255);
		love.graphics.rectangle("fill", v:getDrawbox());
		love.graphics.setColor(255, 255, 255, 255);
		love.graphics.rectangle("line", v:getDrawbox());
	end
end

function updateGame(dt)
	local diff = love.timer.getTime() - tick;
	if diff > interval then
		table.insert(ents, brick());
		tick = love.timer.getTime();
	end
	
	if table.getn(ents) > limit then
		love.audio.stop();
		bgm = love.audio.newSource("gameover.mp3");
		love.audio.play(bgm);
		state = STATE_GAMEOVER;
	end
end

function inputGame(key)
	if key == cancel then state = STATE_PAUSED end
	if key == confirm then table.insert(ents, brick()) end
end

function mouseGame(x, y, btn)
	-- check if player clicked a brick
	local destroy = false;
	local key = 0;
	
	for k, v in pairs(ents) do
		-- destroy = v:checkHit(x, y);
		-- print(destroy);
		-- if destroy then
			-- key = k;
			-- break;
		-- end
		if v:checkHit(x, y) then
			table.remove(ents, k);
			score = score + 1;
			if score >= limit * stage then
				interval = interval * diffMod;
				stage = stage + 1;
			end
		end
	end
end

-- STATE_GAMEOVER

function drawGameOver()
	love.graphics.setColor(255, 255, 255, 255);
	love.graphics.printf("GAME OVER!\nYOUR SCORE: " .. score .. "\n\n\nPRESS " .. string.upper(confirm) .. " TO START AGAIN!", 0, height / 2, width, "center", 0);
end

function updateGameOver(dt)
	-- noop
end

function inputGameOver(key)
	if key == confirm then
		love.audio.stop();
		init();
		state = STATE_INGAME;
	elseif key == cancel then
		love.event.quit();
	end
end

function mouseGameOver(x, y, btn)
	-- noop
end

-- STATE_PAUSED

function drawPaused()
	local pause = "PAUSED\n";
	pause = pause .. string.upper(confirm);
	if sys ~= "Android" then
		pause = pause .. " OR CLICK TO CONTINUE\n";
	else
		pause = pause .. " OR TAP TO CONTINUE\n";
	end
	
	pause = pause .. string.upper(cancel) .. " TO QUIT";
	
	love.graphics.setColor(255, 255, 255, 255);
	love.graphics.printf(pause, 0, height / 2, width, "center", 0);
end

function updatePaused(dt)
	--noop
end

function inputPaused(key)
	if key == cancel then
		love.event.quit();
	else
		state = STATE_INGAME;
	end
end

function mousePaused(x, y, btn)
	state = STATE_INGAME;
end